#include <string.h>

#include "tpro.h"

#include "tpro_capabilities.h"
#include "tpro_macros.h"

/*
** Subsytem Product IDs
*/

#define TPRO_DEVID_7000             ( 0x7000 )
#define TPRO_DEVID_7100             ( 0x7100 )
#define TPRO_DEVID_TSAT             ( 0x9070 )
#define TPRO_DEVID_TSAT_HB1PPS      ( 0x9072 )
#define TPRO_DEVID_TSAT_FXA         ( TPRO_DEVID_TSAT_HB1PPS )
#define TPRO_DEVID_TPRO             ( 0x9050 )
#define TPRO_DEVID_TPRO_M           ( 0x9051 )
#define TPRO_DEVID_TPRO_HB1PPS      ( 0x9052 )
#define TPRO_DEVID_TPRO_FXB         ( TPRO_DEVID_TPRO_HB1PPS )
#define TPRO_DEVID_TPRO_LOR1        ( TPRO_DEVID_TPRO_HB1PPS )
#define TPRO_DEVID_TPRO_M_HB1PPS    ( 0x9053 )

#define TPRO_DEVID_PCI66_TSAT          ( 0x9170 )
#define TPRO_DEVID_PCI66_TSAT_HB1PPS   ( 0x9172 )
#define TPRO_DEVID_PCI66_TSAT_FXA      ( TPRO_DEVID_PCI66_TSAT_HB1PPS )
#define TPRO_DEVID_PCI66_TPRO          ( 0x9150 )
#define TPRO_DEVID_PCI66_TPRO_M        ( 0x9151 )
#define TPRO_DEVID_PCI66_TPRO_HB1PPS   ( 0x9152 )
#define TPRO_DEVID_PCI66_TPRO_FXB      ( TPRO_DEVID_PCI66_TPRO_HB1PPS )
#define TPRO_DEVID_PCI66_TPRO_LOR1     ( TPRO_DEVID_PCI66_TPRO_HB1PPS )
#define TPRO_DEVID_PCI66_TPRO_M_HB1PPS ( 0x9153 )

#define TSYNC_DEVID    ( 0x8000 )

typedef struct {
#define TPRO_X(property) tpro_boolean_t property;
    TPRO_CAPABILITIES_LIST
#undef TPRO_X
} board_capabilities_t;

unsigned char check_boardIsTsync(TPRO_BoardObj *hnd,
                                 board_capabilities_t *capabilities) {

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardIsTsync = TPRO_TRUE;
    
    return TPRO_SUCCESS;
}

unsigned char check_boardHasFPGA(TPRO_BoardObj *hnd,
                                 board_capabilities_t *capabilities) {

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardHasFPGA = TPRO_TRUE;
    
    return TPRO_SUCCESS;
}

unsigned char check_boardSupportsGPSHardware(TPRO_BoardObj *hnd,
                                            board_capabilities_t *capabilities) {

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardSupportsGPSHardware = TPRO_TRUE;

    return TPRO_SUCCESS;
}

unsigned char check_boardSyncsToPulseOnly(TPRO_BoardObj *hnd, 
                                          board_capabilities_t *capabilities) {
    
    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardSyncsToPulseOnly = TPRO_FALSE;
    return TPRO_SUCCESS;
}

unsigned char check_boardSupportsGregorianDate(TPRO_BoardObj *hnd,
                                               board_capabilities_t *capabilities) {

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardSupportsGregorianDate = TPRO_TRUE;

    return TPRO_SUCCESS;
}

unsigned char check_boardSupportsHighRateHeartbeat(TPRO_BoardObj *hnd,
                                                   board_capabilities_t *capabilities) {
    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    capabilities->boardSupportsHighRateHeartbeat = TPRO_TRUE;

    return TPRO_SUCCESS;
}

unsigned char tpro_getCapabilities(TPRO_BoardObj *hnd,
                                   board_capabilities_t *capabilities) {
    static TPRO_BoardObj *previousHnd = NULL;
    static board_capabilities_t knownCapabilities;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(capabilities);

    /* Skip assessing the board if we've still got its capabilities in
     * our own struct. */
    if (hnd != previousHnd) {

#define TPRO_X(property) CHECK(check_##property(hnd, &knownCapabilities), TPRO_SUCCESS);
        TPRO_CAPABILITIES_LIST
#undef TPRO_X

        previousHnd = hnd;
    }
    
    /* Copy what we know */
    memcpy(capabilities, &knownCapabilities, sizeof(board_capabilities_t));
    return TPRO_SUCCESS;
}

#define TPRO_X(property)                                                \
unsigned char TPRO_##property(TPRO_BoardObj *hnd,                       \
                              tpro_boolean_t *result) {                 \
                                                                        \
    board_capabilities_t capabilities;                                  \
                                                                        \
    CHECK_HANDLE(hnd);                                                  \
    CHECK_NOT_NULL(result);                                             \
                                                                        \
    CHECK(tpro_getCapabilities(hnd, &capabilities), TPRO_SUCCESS);      \
                                                                        \
    *result = capabilities.property;                                    \
    return TPRO_SUCCESS;                                                \
}
TPRO_CAPABILITIES_LIST
#undef TPRO_X

