#include "tpro.h"
#include "tpro_types.h"
#include "tpro_macros.h"
#include "tpro_error_codes.h"


tpro_boolean_t tpro_isLeapYear(unsigned int year) {

    /* Zero is a magical, invalid year which can only be achieved by
     * resetting the firmware. */
    if (year == 0) return TPRO_FALSE;

    if (year % 400 == 0) return TPRO_TRUE;
    if (year % 100 == 0) return TPRO_FALSE;
    if (year % 4 == 0) return TPRO_TRUE;

    return TPRO_FALSE;
}

unsigned char tpro_validateTime(TPRO_BoardObj *hnd, TPRO_TimeObj *time) {

    TPRO_DateObj date;

    CHECK(TPRO_getDate(hnd, &date), TPRO_SUCCESS);
    tpro_boolean_t isLeapYear = tpro_isLeapYear(date.year);

    unsigned int maxDay = (isLeapYear == TPRO_TRUE) ? 366 : 365;

    if (time->days > maxDay) { return TPRO_DAY_PARM_ERR; }
    if (time->hours > 23)    { return TPRO_HOUR_PARM_ERR; }
    if (time->minutes > 59)  { return TPRO_MIN_PARM_ERR; }
    if (time->seconds > 59)  { return TPRO_SEC_PARM_ERR; }

    return TPRO_SUCCESS;
}

unsigned char tpro_validateMatchTime(TPRO_MatchObj *match) {

    if (match->days > 366)      { return TPRO_DAY_PARM_ERR; }
    if (match->hours > 23)      { return TPRO_HOUR_PARM_ERR; }
    if (match->minutes > 59)    { return TPRO_MIN_PARM_ERR; }
    if (match->seconds >= 60.0) { return TPRO_SEC_PARM_ERR; }

    if ((match->matchType != MATCH_TIME_START) &&
        (match->matchType != MATCH_TIME_STOP)) {
        return TPRO_MATCH_PARM_ERR;
    }

    return TPRO_SUCCESS;
}

unsigned char tpro_DOYToDate(
    unsigned int dayOfYear, unsigned int year,
    unsigned char* month, unsigned char* day)
{
    unsigned short daysPerMonth[] = { 
        31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0xffff - 366 };

    unsigned int iMonth = 0; 
    unsigned int firstDOYThisMonth = 1;
    unsigned int lastDOYThisMonth = daysPerMonth[0];
    unsigned int monthFound = 0;
    
    if (tpro_isLeapYear(year) == TPRO_TRUE) { daysPerMonth[1] += 1; }

    while (!monthFound) {
        if (dayOfYear > lastDOYThisMonth) {
            firstDOYThisMonth += daysPerMonth[iMonth];
            iMonth++;
            lastDOYThisMonth += daysPerMonth[iMonth];
        }
        else {
            monthFound = 1;
        }
    }

    if (iMonth >= 12) {
        iMonth = 12;
    }

    *month = iMonth + 1;
    *day = dayOfYear - firstDOYThisMonth + 1;

  return (0);
}

