
  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  tpro_boolean_t bIsTsync;
  CHECK(TPRO_boardIsTsync(hnd, &bIsTsync),
        TPRO_SUCCESS);

  if(bIsTsync == TPRO_TRUE)
  {
    TSYNC_BoardHandle handle = (TSYNC_BoardHandle) hnd;
    TSYNC_ERROR err;
    unsigned int nCount = 0;

    while(1)
    {
        //check if a  timestamp is in the legacy queue
        err = TSYNC_HW_getTsCount(
            handle,
            TMSTMP_SRC_HOST_AND_GPI_0,
            &nCount);
        if (err != TSYNC_SUCCESS)
        {
            printf(" *** Error: %s.\n", tsync_strerror(err));
            return TPRO_COMM_ERR;
        }

        //if there is a timestamp there, exit this loop
        if(nCount > 0)
        {
            break;
        }

        // enable the interrupt
        err = TSYNC_HW_setIntMask(
            handle,
            INT_TMSTMP,
            0,
            0);
        if (err != TSYNC_SUCCESS)
        {
            printf(" *** Error: %s.\n", tsync_strerror(err));
            return TPRO_COMM_ERR;
        }

        // wait for any timestamp
        err = TSYNC_waitFor(
            handle,
            INT_TMSTMP,
            0);
        if (err != TSYNC_SUCCESS)
        {
            printf(" *** Error: %s.\n", tsync_strerror(err));
            return TPRO_COMM_ERR;
        }
    }

    //get the legacy timestamp
    TSYNC_HWTimeObj pObj;
    err = TSYNC_HW_getTsSingle(
        handle,
        TMSTMP_SRC_HOST_AND_GPI_0,
        &pObj);
    if (err != TSYNC_SUCCESS)
    {
        printf(" *** Error: %s.\n", tsync_strerror(err));
        return TPRO_COMM_ERR;
    }

    waitp->seconds = pObj.time.seconds;
    waitp->minutes = pObj.time.minutes;
    waitp->hours = pObj.time.hours;
    waitp->days = pObj.time.doy;

    return (TPRO_SUCCESS);
  }
