#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:t:c"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    int             imageType = -1;
    int             bCalcCRC = 0;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 't' :
                imageType = atoi(optarg);
                break;
            case 'c' :
                bCalcCRC = 1;
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc || imageType >= UL_IMG_NUM_ITEMS )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    int i;
    int loopStart = imageType != -1 ? imageType : 0;
    int loopEnd = imageType != -1 ? imageType + 1 : UL_IMG_NUM_ITEMS;
    for(i = loopStart; i < loopEnd; i++)
    {
        TSYNC_ULImageObj pObj;
        pObj.image = (UL_IMG)i;
        printf("Flash Manager Test for Image Type: %u\n", pObj.image);
        
        //Flash Manager Get CRC Test
        {
            TSYNC_FSCRCObj pObj2;
        
            TSYNC_ERROR err = TSYNC_FS_getCrc(hnd, &pObj, &pObj2);
            
            if (err == TSYNC_SUCCESS)
            {
                printf(" Stored CRC: 0x%08x\n",
                    pObj2.crc);
            }
            else
            {
                printf(" Stored CRC: %s.\n", tsync_strerror(err));
            }
        }
        
        
        //Flash Manager Calc CRC Test
        {
            if(bCalcCRC)
            {
                TSYNC_FSCRCObj pObj2;
            
                TSYNC_ERROR err = TSYNC_FS_calcCrc(hnd, &pObj, &pObj2);
                
                if (err == TSYNC_SUCCESS)
                {
                    printf(" Calculated CRC: 0x%08x\n",
                        pObj2.crc);
                }
                else
                {
                    printf(" Stored CRC: %s.\n", tsync_strerror(err));
                }
            }
            else
            {
                    printf(" Calculated CRC: Skipped\n");
            }
        }
        
        //Flash Manager Get Header Test
        {
            TSYNC_ULImageHeaderObj pObj2;
        
            TSYNC_ERROR err = TSYNC_FS_getHeader(hnd, &pObj, &pObj2);
            
            if (err == TSYNC_SUCCESS)
            {
                printf(" Header:\n  Marker: 0x%08x\n  Type: %u\n  Length: %u\n",
                    pObj2.mark, pObj2.type, pObj2.len);
            }
            else
            {
                printf(" Header: %s.\n", tsync_strerror(err));
            }
        }
        
        //Flash Manager Get Version Test
        {
            TSYNC_FSVersionObj pObj2;
    
            TSYNC_ERROR err = TSYNC_FS_getVersion(hnd, &pObj, &pObj2);
            
            if (err == TSYNC_SUCCESS)
            {
                char buff[5];
                memcpy(buff, pObj2.version, 4);
                buff[4] = '\0';
                printf(" Version: %s\n", buff);
            }
            else
            {
                printf(" Version: %s.\n", tsync_strerror(err));
            }
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\t-c   force calculation of CRC (off by default)\n");
    (void)printf("\n");
}
