#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get GPO Signature Control
    {
        OD_PIN gpo = OD_PIN_0;
        SIG_CTL sig;
        
        printf(" Get GPO Signature Control (%u)\n", gpo);
        err = TSYNC_GO_getSigCtrl(hnd, gpo, &sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPO Signature Control: %u\n", sig);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO Signature Control
    {
        OD_PIN gpo = OD_PIN_0;
        SIG_CTL sig = SIG_CTL_SYNC;
        
        printf(" Set GPO Signature Control (%u, %u)\n", gpo, sig);
        err = TSYNC_GO_setSigCtrl(hnd, gpo, sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPO Output Enable
    {
        OD_PIN gpo = OD_PIN_0;
        int bEnable;
        
        printf(" Get GPO Output Enable (%u)\n", gpo);
        err = TSYNC_GO_getEnable(hnd, gpo, &bEnable);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPO Output Enable: %u\n", bEnable);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO Output Enable
    {
        OD_PIN gpo = OD_PIN_0;
        int bEnable = 0;
        
        printf(" Set GPO Output Enable (%u, %u)\n", gpo, bEnable);
        err = TSYNC_GO_setEnable(hnd, gpo, bEnable);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPO Value
    {
        OD_PIN gpo = OD_PIN_0;
        int bValue;
        
        printf(" Get GPO Value (%u)\n", gpo);
        err = TSYNC_GO_getValue(hnd, gpo, &bValue);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPO Value: %u\n", bValue);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPO Mode
    {
        OD_PIN gpo = OD_PIN_0;
        OD_MODE mode;
        
        printf(" Get GPO Mode (%u)\n", gpo);
        err = TSYNC_GO_getMode(hnd, gpo, &mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPO Mode: %u\n", mode);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO Mode
    {
        OD_PIN gpo = OD_PIN_0;
        OD_MODE mode = OD_MODE_DIRECT;
        
        printf(" Set GPO Mode (%u, %u)\n", gpo, mode);
        err = TSYNC_GO_setMode(hnd, gpo, mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPO DVM Value
    {
        OD_PIN gpo = OD_PIN_0;
        int bValue;
        
        printf(" Get GPO DVM Value (%u)\n", gpo);
        err = TSYNC_GO_getDvmValue(hnd, gpo, &bValue);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPO DVM Value: %u\n", bValue);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO DVM Value
    {
        OD_PIN gpo = OD_PIN_0;
        int bValue = 0;
        
        printf(" Set GPO DVM Value (%u, %u)\n", gpo, bValue);
        err = TSYNC_GO_setDvmValue(hnd, gpo, bValue);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO Match Enable
    /*{
        OD_PIN gpo = OD_PIN_0;
        TSYNC_GPOMatchObj pObj;
        //pObj. = //set some values!
        
        printf(" Set GPO Match Enable (%u)\n", gpo);
        err = TSYNC_GO_setMatchEnable(hnd, gpo, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Get GPO Match Enable
    {
        OD_PIN gpo = OD_PIN_0;
        LEVEL lvl = LEVEL_HIGH;
        int bEnable;
        
        printf(" Get GPO Match %s Enable (%u)\n",
               ((lvl == LEVEL_HIGH) ? "High" : "Low"), gpo);
        err = TSYNC_GO_getMatchEnable(hnd, gpo, lvl, &bEnable);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPO Match Enable:\n"
                    "   bEnable: %u\n",
                    bEnable);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPO Square Wave
    /*{
        OD_PIN gpo = OD_PIN_0;
        TSYNC_GPOSquareObj pObj;
        //pObj. = //set some values!
        
        printf(" Set GPO Square Wave (%u)\n", gpo);
        err = TSYNC_GO_setSquareWave(hnd, gpo, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Get GPO Square Wave
    {
        OD_PIN gpo = OD_PIN_0;
        TSYNC_GPOSquareObj pObj;
        
        printf(" Get GPO Square Wave (%u)\n", gpo);
        err = TSYNC_GO_getSquareWave(hnd, gpo, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPO Square Wave:\n"
                    "   off: %u\n"
                    "   per: %u\n"
                    "   dc: %u\n"
                    "   ae: %u\n",
                    pObj.off,
                    pObj.per,
                    pObj.pw,
                    pObj.ae);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
