#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get GPS Offset
    {
        unsigned int nInstance = 0;
        int nOffset;
        
        printf(" Get GPS Offset\n");
        err = TSYNC_GR_getOffset(hnd, nInstance, &nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPS Offset: %u\n", nOffset);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Reference Validity
    {
        unsigned int nInstance = 0;
        int bTimeValid;
        int bPpsValid;
        
        printf(" Get GPS Reference Validity\n");
        err = TSYNC_GR_getValidity(hnd, nInstance, &bTimeValid, &bPpsValid);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Reference Validity:\n"
                    "   Time Valid: %u\n"
                    "   PPS Valid: %u\n",
                    bTimeValid,
                    bPpsValid);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Position
    {
        unsigned int nInstance = 0;
        TSYNC_LLAObj pObj;
        
        printf(" Get GPS Position\n");
        err = TSYNC_GR_getPosition(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Position:\n"
                    "   Latitude: %f\n"
                    "   Longitude: %f\n"
                    "   Altitude: %f\n",
                    pObj.lat,
                    pObj.lon,
                    pObj.alt);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPS Position
    {
        unsigned int nInstance = 0;
        TSYNC_LLAObj pObj;
        pObj.lat = 10.5;
        pObj.lon = 50.1234;
        pObj.alt = -5.5;
        
        printf(" Set GPS Position (%f, %f, %f)\n", pObj.lat, pObj.lon, pObj.alt);
        err = TSYNC_GR_setPosition(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Receiver Mode
    {
        unsigned int nInstance = 0;
        GL_MODE mode = GL_MODE_1SAT;
        GL_DYN  dyn  = GL_DYN_LAND;
        
        printf(" Get GPS Receiver Mode and Dynamics\n");
        err = TSYNC_GR_getMode(hnd, nInstance, &mode, &dyn);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPS Receiver Mode: (%u) Dynamics (%u)\n", mode, dyn);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPS Receiver Mode
    {
        unsigned int nInstance = 0;
        GL_MODE mode = GL_MODE_1SAT;
        GL_DYN  dyn  = GL_DYN_LAND;
        
        printf(" Set GPS Receiver Mode (%u) Dynamics (%u)\n", mode, dyn);
        err = TSYNC_GR_setMode(hnd, nInstance, mode, dyn);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Dynamics Mode
    {
        unsigned int nInstance = 0;
        GL_DYN mode;
        
        printf(" Get GPS Dynamics Mode\n");
        err = TSYNC_GR_getDynamics(hnd, nInstance, &mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPS Dynamics Mode: %u\n", mode);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPS Dynamics Mode
    {
        unsigned int nInstance = 0;
        GL_DYN mode = GL_DYN_LAND;
        
        printf(" Set GPS Dynamics Mode (%u)\n", mode);
        err = TSYNC_GR_setDynamics(hnd, nInstance, mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Fix Data
    {
        unsigned int nInstance = 0;
        TSYNC_FixDataObj pObj;
        
        printf(" Get Fix Data\n");
        err = TSYNC_GR_getFixData(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Fix Data:\n"
                    "   Num Sats: %u\n"
                    "   Pos DOP: %f\n"
                    "   Horz DOP: %f\n"
                    "   Vert DOP: %f\n"
                    "   Time DOP: %f\n"
                    "   FOM: %u\n"
                    "   Time FOM: %u\n"
                    "   Horz Err: %u\n"
                    "   Vert Err: %u\n",
                    pObj.nSats,
                    pObj.pdop,
                    pObj.hdop,
                    pObj.vdop,
                    pObj.tdop,
                    pObj.fom,
                    pObj.tfom,
                    pObj.herr,
                    pObj.verr);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Sat Data
    {
        unsigned int nInstance = 0;
        TSYNC_SatDataObj pObj;
        
        printf(" Get Sat Data\n");
        err = TSYNC_GR_getSatData(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Sat Data (0):\n"
                    "   Channel Number: %u\n"
                    "   SVID: %u\n"
                    "   Signal Strength: %u\n"
                    "   Traim: %u\n"
                    "   Infix: %u\n"
                    "   Bit Flags: 0x%08X\n",
                    pObj.info[0].chnum,
                    pObj.info[0].svid,
                    pObj.info[0].str,
                    pObj.info[0].bTraim,
                    pObj.info[0].bInfix,
                    pObj.info[0].flags);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Survery Progress
    {
        unsigned int nInstance = 0;
        unsigned int nProgress;
        
        printf(" Get GPS Survery Progress\n");
        err = TSYNC_GR_getSurveyProg(hnd, nInstance, &nProgress);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPS Survery Progress: %u\n", nProgress);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Manufacturer Model
    {
        unsigned int nInstance = 0;
        TSYNC_ManModObj pObj;
        
        printf(" Get GPS Manufacturer Model\n");
        err = TSYNC_GR_getMfrMdl(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Manufacturer Model:\n"
                    "   Manufacturer: %s\n"
                    "   Model: %s\n",
                    pObj.mfr,
                    pObj.mdl);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Receiver Info
    {
        unsigned int nInstance = 0;
        TSYNC_ReceiverInfoObj pObj;
        
        printf(" Get GPS Receiver Info\n");
        err = TSYNC_GR_getRcvInfo(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Receiver Info:\n"
                    "   Info: %s\n",
                    pObj.info);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPS Custom Message
    {
        unsigned int nInstance = 0;
        TSYNC_CustomMessageObj pObj;
        char msg[] = "This is the GPS Custom Message.";
        pObj.len = sizeof(msg) - 1;
        memcpy(pObj.msg, msg, pObj.len);
        
        printf(" Set GPS Custom Message (%u, %s)\n", pObj.len, msg);
        err = TSYNC_GR_setCustom(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Custom Message
    {
        unsigned int nInstance = 0;
        TSYNC_CustomMessageObj pObj;
        
        printf(" Get GPS Custom Message\n");
        err = TSYNC_GR_getCustom(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  GPS Custom Message:\n"
                    "   Len: %u\n"
                    "   Msg: %s\n",
                    pObj.len,
                    pObj.msg);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPS Number of Instances
    {
        unsigned int nInstances;
        
        printf(" Get GPS Number of Instances\n");
        err = TSYNC_GR_getNumInst(hnd, &nInstances);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPS Number of Instances: %u\n", nInstances);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Delete GPS Stored Position
    {
        unsigned int nInstance = 0;

        printf(" Delete GPS Stored Position\n");
        err = TSYNC_GR_delPos(hnd, nInstance);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
