#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

/********************************************************
**
** Test Module: GetStatus.c
**
** Purpose:     Get the current upgrade status from the
**              board.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    TSYNC_StateObj obj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * get the state
     */
     TSYNC_ERROR err = TSYNC_US_getState(hnd, &obj);
    
    if (err != TSYNC_SUCCESS)
    {
        (void) printf("%s: <%s> %s.\n",
                    pgmname, fullDevName, tsync_strerror(err));

        ret = 1;

    }
    else
    {
        /*
         * success:  print out the results
         */
        (void)printf("<%s> %s: Image Type <%d> Process Step <%d> Completed <%d>\n",
                     fullDevName, pgmname,
                     obj.image,
                     obj.step,
                     obj.complete);
                
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
