#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:sb"

/********************************************************
**
** Test Module: GetTime.c
**
** Purpose:     Get the current time from the board
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    TSYNC_ERROR         rc;
    int                 ret = 0;
    int                 devIdx = DEFAULT_DEV_NUM;
    char                *devName = DEFAULT_DEV;
    int                 optRc;
    char                fullDevName[32];
    char                *pgmname = argv[0];
    ML_TIME_TYPE        timeType = ML_TIME_DOYTIME;
    TSYNC_ERROR         err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                timeType = ML_TIME_SECONDS;
                break;
            case 'b' :
                timeType = ML_TIME_BCDTIME;
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: Could not open <%s>: %s\n",
               pgmname, fullDevName, tsync_strerror(rc));
        return (1);
    }

    //Get Time DOY
    if(timeType == ML_TIME_DOYTIME)
    {
        TSYNC_TimeObj pObj;

        printf(" Get Time\n");
        err = TSYNC_CS_getTime(hnd, &pObj);

        if (err == TSYNC_SUCCESS)
        {
            printf( "  Years: %u\n"
                    "  Day of Year: %u\n"
                    "  Hours: %u\n"
                    "  Minutes: %u\n"
                    "  Seconds: %u\n"
                    "  Nanoseconds: %u\n",
                     pObj.years,
                     pObj.doy,
                     pObj.hours,
                     pObj.minutes,
                     pObj.seconds,
                     pObj.ns);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }

    //Get Time Seconds
    else if(timeType == ML_TIME_SECONDS)
    {
        unsigned int nSeconds;
        unsigned int nNanos;

        printf(" Get Time Seconds\n");
        err = TSYNC_CS_getTimeSec(hnd, &nSeconds, &nNanos);

        if (err == TSYNC_SUCCESS)
        {
            printf( "  Seconds: %u\n"
                    "  Nanos: %u\n",
                    nSeconds,
                    nNanos);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }

    //Get Time BCD
    else if(timeType == ML_TIME_BCDTIME)
    {
        TSYNC_TimeBCDObj pObj;

        printf(" Get Time BCD\n");
        err = TSYNC_CS_getTimeBcd(hnd, &pObj);

        if (err == TSYNC_SUCCESS)
        {
            printf( "  Years: %u (0x%08X)\n"
                    "  Day of Year: %u (0x%08X)\n"
                    "  Hours: %u (0x%08X)\n"
                    "  Minutes: %u (0x%08X)\n"
                    "  Seconds: %u (0x%08X)\n"
                    "  Ms: %u (0x%08X)\n"
                    "  Us: %u (0x%08X)\n",
                     pObj.years, pObj.years,
                     pObj.doy,pObj.doy,
                     pObj.hours,pObj.hours,
                     pObj.minutes,pObj.minutes,
                     pObj.seconds,pObj.seconds,
                     pObj.ms,pObj.ms,
                     pObj.us,pObj.us);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s>: %s\n",
                      pgmname, fullDevName, tsync_strerror(rc) );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
