#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get IRIG Reference Offset
    {
        unsigned int nInstance = 0;
        int nOffset;
        
        printf(" Get IRIG Reference Offset (%u)\n", nInstance);
        err = TSYNC_IR_getOffset(hnd, nInstance, &nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Reference Offset: %u\n", nOffset);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Reference Offset
    {
        unsigned int nInstance = 0;
        int nOffset = 0;
        
        printf(" Set IRIG Reference Offset (%u, %u)\n", nInstance, nOffset);
        err = TSYNC_IR_setOffset(hnd, nInstance, nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Reference Validity
    {
        unsigned int nInstance = 0;
        int bTimeValid;
        int bPpsValid;
        
        printf(" Get IRIG Reference Validity (%u)\n", nInstance);
        err = TSYNC_IR_getValidity(hnd, nInstance, &bTimeValid, &bPpsValid);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  IRIG Reference Validity:\n"
                    "   Time Valid: %u\n"
                    "   PPS Valid: %u\n",
                    bTimeValid,
                    bPpsValid);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Mode
    {
        unsigned int nInstance = 0;
        IL_MODE mode = IL_MODE_AUTO;
        
        printf(" Set IRIG Mode (%u, %u)\n", nInstance, mode);
        err = TSYNC_IR_setMode(hnd, nInstance, mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Mode
    {
        unsigned int nInstance = 0;
        IL_MODE mode;
        
        printf(" Get IRIG Mode (%u)\n", nInstance);
        err = TSYNC_IR_getMode(hnd, nInstance, &mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Mode: %u\n", mode);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Format
    {
        unsigned int nInstance = 0;
        IL_FMT format = IL_FMT_IRIG_A;
        
        printf(" Set IRIG Format (%u, %u)\n", nInstance, format);
        err = TSYNC_IR_setFormat(hnd, nInstance, format);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Frequency
    {
        unsigned int nInstance = 0;
        IL_FRQ freq = IL_FRQ_100HZ;
        
        printf(" Set IRIG Frequency (%u, %u)\n", nInstance, freq);
        err = TSYNC_IR_setFreq(hnd, nInstance, freq);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Coded Expression
    {
        unsigned int nInstance = 0;
        IL_CE ce = IL_CE_BCDT_CF_SBS;
        
        printf(" Set IRIG Coded Expression (%u, %u)\n", nInstance, ce);
        err = TSYNC_IR_setCodedExpr(hnd, nInstance, ce);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Control Field
    {
        unsigned int nInstance = 0;
        IL_CF cf = IL_CF_200_04;
        
        printf(" Set IRIG Control Field (%u, %u)\n", nInstance, cf);
        err = TSYNC_IR_setCtrlField(hnd, nInstance, cf);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Format
    {
        unsigned int nInstance = 0;
        IL_FMT format;
        
        printf(" Get IRIG Format (%u)\n", nInstance);
        err = TSYNC_IR_getFormat(hnd, nInstance, &format);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Format: %u\n", format);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Modulation
    {
        unsigned int nInstance = 0;
        IL_MOD mod;
        
        printf(" Get IRIG Modulation (%u)\n", nInstance);
        err = TSYNC_IR_getMod(hnd, nInstance, &mod);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Modulation: %u\n", mod);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Frequency
    {
        unsigned int nInstance = 0;
        IL_FRQ freq;
        
        printf(" Get IRIG Frequency (%u)\n", nInstance);
        err = TSYNC_IR_getFreq(hnd, nInstance, &freq);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Frequency: %u\n", freq);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Coded Expression
    {
        unsigned int nInstance = 0;
        IL_CE ce;
        
        printf(" Get IRIG Coded Expression (%u)\n", nInstance);
        err = TSYNC_IR_getCodedExpr(hnd, nInstance, &ce);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Coded Expression: %u\n", ce);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Control Field
    {
        unsigned int nInstance = 0;
        IL_CF cf;
        
        printf(" Get IRIG Control Field (%u)\n", nInstance);
        err = TSYNC_IR_getCtrlField(hnd, nInstance, &cf);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Control Field: %u\n", cf);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Message
    /*{
        unsigned int nInstance = 0;
        TSYNC_IRIGMessageObj pObj;
        int i;
        for(i = 0; i < TSYNC_IR_SUBFRAME_NUM; i++)
        {
            pObj.subframes[i] = i;
        }
        
        printf(" Set IRIG Message (%u)\n", nInstance);
        err = TSYNC_IR_setMessage(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Get IRIG Message
    {
        unsigned int nInstance = 0;
        TSYNC_IRIGMessageObj pObj;
        
        printf(" Get IRIG Message (%u)\n", nInstance);
        err = TSYNC_IR_getMessage(hnd, nInstance, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  IRIG Message:\n");
            int i;
            for(i = 0; i < TSYNC_IR_SUBFRAME_NUM; i++)
            {
                printf("   Sub P%u: %u\n", i, pObj.subframes[i]);
            }
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Number of Instances
    {
        unsigned int nInstances;
        
        printf(" Get IRIG Number of Instances\n");
        err = TSYNC_IR_getNumInst(hnd, &nInstances);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Number of Instances: %u\n", nInstances);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
