#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get GPI Value
    {
        ID_PIN index = ID_PIN_0;
        int bEnabled;
        
        printf(" Get GPI Value (%u)\n", index);
        err = TSYNC_GI_getValue(hnd, index, &bEnabled);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPI Value: %u\n", bEnabled);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPI Edge
    {
        ID_PIN index = ID_PIN_0;
        EDGE edge = EDGE_RISING;
        
        printf(" Set GPI Edge (%u, %u)\n", index, edge);
        err = TSYNC_GI_setEdge(hnd, index, edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPI Edge
    {
        ID_PIN index = ID_PIN_0;
        EDGE edge;
        
        printf(" Get GPI Edge (%u)\n", index);
        err = TSYNC_GI_getEdge(hnd, index, &edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPI Edge: %u\n", edge);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set GPI Timestamp Enable
    {
        ID_PIN index = ID_PIN_0;
        int bEnable = 0;
        
        printf(" Set GPI Timestamp Enable (%u, %u)\n", index, bEnable);
        err = TSYNC_GI_setTsEnable(hnd, index, bEnable);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPI Timestamp Enable
    {
        ID_PIN index = ID_PIN_0;
        int bEnable;
        
        printf(" Get GPI Timestamp Enable (%u)\n", index);
        err = TSYNC_GI_getTsEnable(hnd, index, &bEnable);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPI Timestamp Enable: %u\n", bEnable);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get GPI Number of Instances
    {
        unsigned int nInstances;
        
        printf(" Get GPI Number of Instances\n");
        err = TSYNC_GI_getNumInst(hnd, &nInstances);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  GPI Number of Instances: %u\n", nInstances);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
