#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc)
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    TSYNC_MeterHandle pObj;
    //Osc Mon Register meter
    {
        //TSYNC_FSCRCObj pObj2;
    
        TSYNC_ERROR err = TSYNC_XS_register(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(" Meter handle registered: 0x%08x\n", pObj.hnd);
        }
        else
        {
            printf(" Meter handle register error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Osc Mon Set Window Size
    {
        TSYNC_MeterWinSizeObj pObj2;
        pObj2.hnd = pObj.hnd;
        pObj2.size = 5;
    
        TSYNC_ERROR err = TSYNC_XS_setWindowSize(hnd, &pObj2);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(" Set Meter window size: 0x%08x\n", pObj2.size);
        }
        else
        {
            printf(" Meter window size error: %s.\n", tsync_strerror(err));
        }
        //sleep(1);
    }
    
    //Osc Mon Get Window Size
    {
        TSYNC_MeterWinSizeObj pObj2;
        pObj2.hnd = pObj.hnd;
    
        TSYNC_ERROR err = TSYNC_XS_getWindowSize(hnd, &pObj2);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(" Get Meter window size: 0x%08x\n", pObj2.size);
        }
        else
        {
            printf(" Meter window size error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Osc Mon Send Meter Command
    {
        TSYNC_MeterCommandObj pObj2;
        pObj2.hnd = pObj.hnd;
        pObj2.command = XS_CMD_START;
    
        TSYNC_ERROR err = TSYNC_XS_meterCmd(hnd, &pObj2);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(" Success: XS_CMD_START\n");
        }
        else
        {
            printf(" XS_CMD_START error: %s.\n", tsync_strerror(err));
        }
    }
    
    while(1)
    {
        TSYNC_SLEEP(1);
        
        //Osc Mon Get Meter Data
        {
            TSYNC_MeterDataObj pObj2;
            pObj2.hnd = pObj.hnd;
        
            TSYNC_ERROR err = TSYNC_XS_getMeterData(hnd, &pObj2);
            
            if (err == TSYNC_SUCCESS)
            {
                if(pObj2.state == XS_STATE_STOPPED)
                {
                    printf(" Get Meter Data:"
                            "\n  State: 0x%08x"
                            "\n  Win Size: 0x%08x"
                            "\n  Elapsed: 0x%08x"
                            "\n  frAccum: %f"
                            "\n  frPrev: %f"
                            "\n  phStart: %f"
                            "\n  phAccum: %f"
                            "\n  phPrev: %f"
                            "\n",
                            pObj2.state,
                            pObj2.size,
                            pObj2.elapsed,
                            pObj2.frAccum,
                            pObj2.frPrev,
                            pObj2.phStart,
                            pObj2.phAccum,
                            pObj2.phPrev);
                    break;
                }
                else
                {
                    printf(" Progress: %u/%u s\n",
                            pObj2.elapsed,
                            pObj2.size);
                }
            }
            else
            {
                printf(" Get Meter Data error: %s.\n", tsync_strerror(err));
                break;
            }
        }
    }
    
    //Osc Mon Unregister meter
    {
        TSYNC_ERROR err = TSYNC_XS_unregister(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(" Meter handle unregistered: 0x%08x\n", pObj.hnd);
        }
        else
        {
            printf(" Meter handle unregister error: %s.\n", tsync_strerror(err));
        }
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\n");
}
