#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

void PrintBuffer(
        uint8_t*    pBuffer,
        uint32_t    ulLen)
{
    /* Print the serialized contents of the buffer */
    unsigned int i;
    for(i = 0; i < ulLen; i++)
    {
        if(i % 16 == 0)
        {
            printf("0x%02X: ", i);
        }

        printf("%02X ", pBuffer[i]);

        if(i % 16 == 7)
        {
            printf(" ");
        }
        else if(i % 16 == 15)
        {
            printf("\n");
        }
        
    }
}

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get Oscillator Disciplining State
    {
        int disc;
        
        printf(" Get Oscillator Disciplining State\n");
        err = TSYNC_XO_getDiscState(hnd, &disc);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Oscillator Disciplining State: %d\n", disc);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator Mode
    {
        XO_MODE mode;
        
        printf(" Get Oscillator Mode\n");
        err = TSYNC_XO_getMode(hnd, &mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Oscillator Mode: %u\n", mode);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Oscillator Mode
    {
        XO_MODE mode = XO_MODE_DISC;
        
        printf(" Set Oscillator Mode (%u)\n", mode);
        err = TSYNC_XO_setMode(hnd, mode);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator DAC
    {
        unsigned short dac;
        
        printf(" Get Oscillator DAC\n");
        err = TSYNC_XO_getDac(hnd, &dac);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Oscillator DAC: %u\n", dac);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Oscillator DAC
    {
        unsigned short dac = 0;
        
        printf(" Set Oscillator DAC (%u)\n", dac);
        err = TSYNC_XO_setDac(hnd, dac);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator Alarm
    {
        unsigned int alarm;
        
        printf(" Get Oscillator Alarm\n");
        err = TSYNC_XO_getAlarm(hnd, &alarm);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Oscillator Alarm: %d\n", alarm);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator Serial Number
    {
        TSYNC_SerialNoObj sernum;
        
        printf(" Get Oscillator Serial Number\n");
        err = TSYNC_XO_getSerNum(hnd, &sernum);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Oscillator Serial Number: %s\n", sernum.serno);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator Manufacturer Model
    {
        TSYNC_ManModObj pObj;
        
        printf(" Get Oscillator Manufacturer Model\n");
        err = TSYNC_XO_getMfrMdl(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  Oscillator Manufacturer Model:\n"
                    "   Manufacturer: %s\n"
                    "   Model: %s\n",
                    pObj.mfr,
                    pObj.mdl);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Oscillator Custom Message
    {
        TSYNC_OscCustomMessageObj pObj;
        char msg[] = "This is the Oscillator Custom Message.";
        pObj.len = sizeof(msg) - 1;
        memcpy(pObj.msg, msg, pObj.len);
        
        printf(" Set Oscillator Custom Message (%u, %s)\n", pObj.len, msg);
        err = TSYNC_XO_setMessage(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Oscillator Disciplining Command
    {
        TSYNC_OscDiscObj pObj;
        memset(pObj.cmd, 'j', TSYNC_OSC_DISC_CMD_SIZE);
        memset(pObj.data, 'k', TSYNC_OSC_DISC_DATA_SIZE);
        
        printf(" Set Oscillator Disciplining Command:\n");
        PrintBuffer(pObj.cmd, TSYNC_OSC_DISC_CMD_SIZE);
        PrintBuffer(pObj.data, TSYNC_OSC_DISC_DATA_SIZE);
        err = TSYNC_XO_setCmd(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Oscillator Disciplining Command
    {
        TSYNC_OscDiscObj pObj;
        memset(pObj.cmd, 'j', TSYNC_OSC_DISC_CMD_SIZE);
        
        printf(" Get Oscillator Disciplining Command:\n");
        PrintBuffer(pObj.cmd, TSYNC_OSC_DISC_CMD_SIZE);
        err = TSYNC_XO_getCmd(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
            PrintBuffer(pObj.data, TSYNC_OSC_DISC_DATA_SIZE);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\n");
}
