#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get PPSOutput Output Signature Control
    {
        unsigned int inst = 0;
        SIG_CTL sig;
        
        printf(" Get PPSOutput Output Signature Control\n");
        err = TSYNC_PP_getSigCtrl(hnd, inst, &sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPSOutput Output Signature Control: %u\n", sig);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPSOutput Output Signature Control
    {
        unsigned int inst = 0;
        SIG_CTL sig = SIG_CTL_SYNC;
        
        printf(" Set PPSOutput Output Signature Control (%u)\n", sig);
        err = TSYNC_PP_setSigCtrl(hnd, inst, sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPSOutput Output Frequency
    {
        unsigned int inst = 0;
        float freq;
        
        printf(" Get PPSOutput Output Frequency\n");
        err = TSYNC_PP_getFreq(hnd, inst, &freq);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPSOutput Output Frequency: %f\n", freq);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Output Offset
    {
        unsigned int inst = 0;
        int nOffset;
        
        printf(" Get PPS Output Offset\n");
        err = TSYNC_PP_getOffset(hnd, inst, &nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Output Offset: %u\n", nOffset);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPS Output Offset
    {
        unsigned int inst = 0;
        int nOffset = 0;
        
        printf(" Set PPS Output Offset (%u)\n", nOffset);
        err = TSYNC_PP_setOffset(hnd, inst, nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Output Edge
    {
        unsigned int inst = 0;
        EDGE edge;
        
        printf(" Get PPS Output Edge\n");
        err = TSYNC_PP_getEdge(hnd, inst, &edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Output Edge: %u\n", edge);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPS Output Edge
    {
        unsigned int inst = 0;
        EDGE edge = EDGE_FALLING;
        
        printf(" Set PPS Output Edge (%u)\n", edge);
        err = TSYNC_PP_setEdge(hnd, inst, edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Output Pulse Width
    {
        unsigned int inst = 0;
        unsigned int pw;
        
        printf(" Get PPS Output Pulse Width\n");
        err = TSYNC_PP_getPulseWidth(hnd, inst, &pw);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Output Pulse Width: %u\n", pw);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPS Output Pulse Width
    {
        unsigned int inst = 0;
        unsigned int pw = 0;
        
        printf(" Set PPS Output Pulse Width (%u)\n", pw);
        err = TSYNC_PP_setPulseWidth(hnd, inst, pw);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
