#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get PPS Reference Offset
    {
        unsigned int nInstance = 0;
        int nOffset;
        
        printf(" Get PPS Reference Offset (%u)\n", nInstance);
        err = TSYNC_PR_getOffset(hnd, nInstance, &nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Reference Offset: %u\n", nOffset);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPS Reference Offset
    {
        unsigned int nInstance = 0;
        int nOffset = 0;
        
        printf(" Set PPS Reference Offset (%u, %u)\n", nInstance, nOffset);
        err = TSYNC_PR_setOffset(hnd, nInstance, nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Reference Edge
    {
        unsigned int nInstance = 0;
        EDGE edge;
        
        printf(" Get PPS Reference Edge (%u)\n", nInstance);
        err = TSYNC_PR_getEdge(hnd, nInstance, &edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Reference Edge: %u\n", edge);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set PPS Reference Edge
    {
        unsigned int nInstance = 0;
        EDGE edge = EDGE_RISING;
        
        printf(" Set PPS Reference Edge (%u, %u)\n", nInstance, edge);
        err = TSYNC_PR_setEdge(hnd, nInstance, edge);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Reference Validity
    {
        unsigned int nInstance = 0;
        int bTimeValid;
        int bPpsValid;
        
        printf(" Get PPS Reference Validity (%u)\n", nInstance);
        err = TSYNC_PR_getValidity(hnd, nInstance, &bTimeValid, &bPpsValid);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  PPS Reference Validity:\n"
                    "   Time Valid: %u\n"
                    "   PPS Valid: %u\n",
                    bTimeValid,
                    bPpsValid);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get PPS Number of Instances
    {
        unsigned int nInstances;
        
        printf(" Get PPS Number of Instances\n");
        err = TSYNC_PR_getNumInst(hnd, &nInstances);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  PPS Number of Instances: %u\n", nInstances);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
