#include "example.h"

static void usage( char *pgmName );
static void PrintReferenceTable(TSYNC_BoardHandle hnd);

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Best Reference
    /*{
        printf(" Get Best Reference\n");
        TSYNC_TableEntryObj pObj;
        err = TSYNC_RS_getBestRef(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            //printf( " Success!\n");
            char cNone[] = "----";
                printf( "\tEnabl\t"
                        "Prior\t"
                        "Time\t"
                        "PPS\t\n");

                printf( "\t%u\t"
                        "%u\t"
                        "%s\t"
                        "%s\t\n",
                        pObj.enab,
                        pObj.prio,
                        pObj.time[0] != '\0' ? pObj.time : cNone,
                        pObj.pps[0] != '\0' ? pObj.pps : cNone);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Get Indexed Reference
    /*{
        printf(" Get Indexed Reference\n");
        TSYNC_TableEntryObj pObj;
        err = TSYNC_RS_getEntry(hnd, 1, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            //printf( " Success!\n");
            char cNone[] = "----";
                printf( "\tEnabl\t"
                        "Prior\t"
                        "Time\t"
                        "PPS\t\n");

                printf( "\t%u\t"
                        "%u\t"
                        "%s\t"
                        "%s\t\n",
                        pObj.enab,
                        pObj.prio,
                        pObj.time[0] != '\0' ? pObj.time : cNone,
                        pObj.pps[0] != '\0' ? pObj.pps : cNone);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Reset to factory default
    {
        printf(" Reset to factory default\n");
        err = TSYNC_RS_setFactDef(hnd);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //Add Reference
    {
        printf(" Add Reference\n");
        TSYNC_TableEntryObj pObj;
        pObj.enab = 0;
        pObj.prio = 1;
        memcpy(pObj.time, "host", 4);
        memcpy(pObj.pps, "epp0", 4);
        err = TSYNC_RS_addEntry(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //Save user table
    /*{
        printf(" Save user table\n");
        err = TSYNC_RS_saveUserDef(hnd);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);*/
    
    //Set priority
    /*{
        printf(" Set priority\n");
        err = TSYNC_RS_setPriority(hnd, 3, 5);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //Get priority
    {
        printf(" Get priority\n");
        unsigned int result;
        err = TSYNC_RS_getPriority(hnd, 3, &result);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Priority: %u\n", result);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //Set enabled
    {
        printf(" Set enabled\n");
        err = TSYNC_RS_setEnable(hnd, 3, 1);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //Get enabled
    {
        printf(" Get enabled\n");
        unsigned int result;
        err = TSYNC_RS_getEnable(hnd, 3, &result);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Enabled: %u\n", result);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);*/
    
    //Delete table entry
    {
        printf(" Delete table entry\n");
        err = TSYNC_RS_deleteEntry(hnd, 3);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( " Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    PrintReferenceTable(hnd);
    
    //State Table
    /*{
        printf(" Get State Table\n");
        TSYNC_ReferenceStateTableObj pObj;
        TSYNC_ERROR err = TSYNC_RS_getStateTable(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            //printf( " Success!\n");
            int i;
            char cNone[] = "----";
            
                printf( "\tIndex\t"
                        "Src\t"
                        "Time V\t"
                        "PPS V\n");
            for(i = 0; i < TSYNC_STATE_TABLE_ENTRY_NUM; i ++)
            {
                printf( "\t%u\t"
                        "%s\t"
                        "%u\t"
                        "%u\n",
                        i,
                        pObj.rows[i].source[0] != '\0' ? pObj.rows[i].source : cNone,
                        pObj.rows[i].timeValid,
                        pObj.rows[i].ppsValid);
            }
            
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void PrintReferenceTable(TSYNC_BoardHandle hnd)
{
    //Table
    {
        printf(" Get Reference Table (RS_TT_CURRENT)\n");
        TSYNC_TableTypeObj pObj;
        pObj.type = RS_TT_CURRENT;
        TSYNC_ReferenceTableObj pObj2;
        TSYNC_ERROR err = TSYNC_RS_getTable(hnd, &pObj, &pObj2);
        
        if (err == TSYNC_SUCCESS)
        {
            //printf( " Success!\n");
            int i;
            char cNone[] = "----";
            
                printf( "\tIndex\t"
                        "Enabl\t"
                        "Prior\t"
                        "Time\t"
                        "PPS\t\n");
            for(i = 0; i < TSYNC_TABLE_ENTRY_NUM; i ++)
            {
                printf( "\t%u\t"
                        "%u\t"
                        "%u\t"
                        "%s\t"
                        "%s\t\n",
                        i,
                        pObj2.rows[i].enab,
                        pObj2.rows[i].prio,
                        pObj2.rows[i].time[0] != '\0' ? pObj2.rows[i].time : cNone,
                        pObj2.rows[i].pps[0] != '\0' ? pObj2.rows[i].pps : cNone);
            }
            
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
