#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

void PrintBuffer(
        uint8_t*    pBuffer,
        uint32_t    ulLen)
{
    /* Print the serialized contents of the buffer */
    unsigned int i;
    for(i = 0; i < ulLen; i++)
    {
        if(i % 16 == 0)
        {
            printf("0x%02X: ", i);
        }

        printf("%02X ", pBuffer[i]);

        if(i % 16 == 7)
        {
            printf(" ");
        }
        else if(i % 16 == 15)
        {
            printf("\n");
        }
        
    }
}

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Shared Memory Reset
    {
        MS_DI_INDEX index = MS_DI_NMEA_GGA;
        
        printf(" Shared Memory Reset (%u)\n", index);
        err = TSYNC_MS_reset(hnd, index);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
      
    //Set Shared Memory Data
    {
        MS_DI_INDEX index = MS_DI_NMEA_GGA;
        TSYNC_SharedMemoryObj pObj;
        pObj.seqNum = 0;
        memset(pObj.data, '\0', 256);
        
        printf(" Set Shared Memory Data (%u)\n", index);
        PrintBuffer((unsigned char*)pObj.data, sizeof(pObj.data));
        err = TSYNC_MS_setData(hnd, index, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get SharedMemory Data
    {
        MS_DI_INDEX index = MS_DI_NMEA_GGA;
        TSYNC_SharedMemoryObj pObj;
        
        printf(" Get Shared Memory Data (%u)\n", index);
        err = TSYNC_MS_getData(hnd, index, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "Success!\n");
            PrintBuffer((unsigned char*)pObj.data, sizeof(pObj.data));
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
