#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Time Scale Test
    {
        printf(" Time Scale Set\n");
        TSYNC_TimeScaleObj pObj;
        pObj.scale = ML_TIME_SCALE_UTC;
        err = TSYNC_CS_setTimeScale(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
        
        printf(" Time Scale Get\n");
        TSYNC_TimeScaleObj pObj2;
        err = TSYNC_CS_getTimeScale(hnd, &pObj2);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!  Scale: %u\n", pObj2.scale);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
        //Time Scale Test
        
        /*printf("*** Set\n");
        TSYNC_TimeScaleObj pObj;
        pObj.scale = 1;
        CHECK_SUCCESS(TSYNC_CS_setTimeScale(hnd, &pObj))*/
        
        /*printf("*** Get\n");
        TSYNC_TimeScaleObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getTimeScale(hnd, &pObj2))
        printf("%u\n", pObj2.scale);*/
        
        //Time Scale Offset Test
        
        /*printf("*** Set\n");
        TSYNC_TimeScaleOffsetObj pObj;
        pObj.scale = 2;
        pObj.offset = 3;
        CHECK_SUCCESS(TSYNC_CS_setTimeScaleOff(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeScaleOffsetObj pObj2;
        pObj2.scale = 2;
        CHECK_SUCCESS(TSYNC_CS_getTimeScaleOff(hnd, &pObj2))
        printf("%u, %u\n", pObj2.scale, pObj2.offset);*/
        
        //Time Subsec Adjust Test
        
        /*printf("*** Set\n");
        TSYNC_TimeSubsecAdjObj pObj;
        pObj.adjust = 5;
        CHECK_SUCCESS(TSYNC_CS_subsecAdj(hnd, &pObj))*/
        
        //Time Discontinuity Test
        
        /*printf("*** Set\n");
        TSYNC_TimeDiscontObj pObj;
        pObj.new.years = 2008;
        pObj.new.doy = 1;
        pObj.new.hours = 1;
        pObj.new.minutes = 1;
        pObj.new.seconds = 5;
        pObj.new.ns = 1;
        pObj.effective.years = 2009;
        pObj.effective.doy = 2;
        pObj.effective.hours = 2;
        pObj.effective.minutes = 2;
        pObj.effective.seconds = 6;
        pObj.effective.ns = 2;
        pObj.bActive = 0;
        CHECK_SUCCESS(TSYNC_CS_setTimeDiscont(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeDiscontObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getTimeDiscont(hnd, &pObj2))
        printf("%u, %u, b:%u\n", pObj2.new.seconds, pObj2.effective.seconds, pObj2.bActive);*/
        
        //Time Leap Second Test
        
        /*printf("*** Set\n");
        TSYNC_TimeLeapSecondObj pObj;
        pObj.offset = 5;
        pObj.utcDate.years = 2008;
        pObj.utcDate.doy = 1;
        pObj.utcDate.hours = 1;
        pObj.utcDate.minutes = 1;
        pObj.utcDate.seconds = 5;
        pObj.utcDate.ns = 1;
        CHECK_SUCCESS(TSYNC_CS_setLeapSec(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeLeapSecondObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getLeapSec(hnd, &pObj2))
        printf("%u, %u\n", pObj2.offset, pObj2.utcDate.years);*/
        
        //Time Zone offset Test
        
        /*printf("*** Set\n");
        TSYNC_TimeZoneOffsetObj pObj;
        pObj.tzOffset = 5;
        CHECK_SUCCESS(TSYNC_CS_setTimeZoneOff(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeZoneOffsetObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getTimeZoneOff(hnd, &pObj2))
        printf("%u\n", pObj2.tzOffset);*/
        
        //Time DST Rule Test
        
        /*printf("*** Set\n");
        TSYNC_TimeDSTRuleObj pObj;
        pObj.offset = 5;
        CHECK_SUCCESS(TSYNC_CS_setDstRule(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeDSTRuleObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getDstRule(hnd, &pObj2))
        printf("%u\n", pObj2.offset);*/
        
        //Year Test
        
        /*printf("*** Set\n");
        TSYNC_TimeYearObj pObj;
        pObj.year = 2010;
        CHECK_SUCCESS(TSYNC_CS_setYear(hnd, &pObj))
        
        printf("*** Get\n");
        TSYNC_TimeYearObj pObj2;
        CHECK_SUCCESS(TSYNC_CS_getYear(hnd, &pObj2))
        printf("%u\n", pObj2.year);*/
        
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
