#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

/* Constant Data */

/* List of characters which are Special in Linux & Unix */
const char EscList[] = { '"', '#', '&', '\'', '(', ')', '*', ',', '.', '/',  \
                         ';', '<', '>', '?',  '`', '|', '~', 0x00}; 


/* Local Function defintions */
static int NeedEsc(char c);


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    unsigned char      num;
    char               req;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: AP_GetReqChar <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Format message */
    index = atoi(argv[2]);

    // Send Get Format transaction
    err = TSYNC_AP_getReqChar(hnd, index, &req);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Convert to integer */
    num = (unsigned int)req;

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        // If the Request Character is greater than a space ' ' 0x20 and less
        //   than or equal to a tilda '~' 0x7E then display the character 
        //   otherwise display a string indicating the hex value 0x## 
        if ((req > ' ') && (req <= '~') && (NeedEsc(req) == 0))
        {
            printf("%c\n", req);
        }
        else
        {
            printf("0x%02x\n", num);
        }
    }

    
    printf("\n  AP (%d) Request Character: (%c) 0x%02x\n", (index), (req), num);
    
    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */


/* Function NeedEsc */
/*   Inputs:  char c : Character to test for linux specialness */
/*   Outputs  int    : 0=ordinary, 1= Linux Special character  */
static int NeedEsc(char c)
{
    int          needEsc = 0;
    unsigned int i;


    for(i = 0; EscList[i] != 0x00; i++)
    {
        if (c == EscList[i])
        {
            needEsc = 1;
            break;
        }
    }

    return needEsc;

}  // End NeedEsc
