#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    UD_CFG             cfg;


    /* If invalid number of arguments... */
    if (argc != 7)
    {
        printf(" Usage: AR_SetUartCfg <device index> <index> <baud>"
               " <data bits> <parity> <stop bits>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Position message */
    index        = atoi(argv[2]);
    cfg.br       = (UD_BR)atoi(argv[3]);
    cfg.numbits  = (UD_DATA)atoi(argv[4]);
    cfg.parity   = (UD_PAR)atoi(argv[5]);
    cfg.stopbits = (UD_STOP)atoi(argv[6]);

    printf("\n");
    printf(" AR (%d) UART Configuration:\n", (index));
    printf("  Baud Rate: %d\n", cfg.br);
    printf("  Data Bits: %d\n", cfg.numbits);
    printf("  Parity:    %d\n", cfg.parity);
    printf("  Stop Bits: %d\n", cfg.stopbits);

    // Send Set Position transaction
    err = TSYNC_AR_setUartCfg(hnd, index, &cfg);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
