#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    TSYNC_TimeLeapSecondObj  ls;


    /* If invalid number of arguments... */
    if (argc != 8)
    {
        printf(" Usage: CS_SetLeapSec <device index> <offset> <hr> <min> <sec> <doy> <yr>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Time message */
    ls.offset          = atoi(argv[2]);
    ls.utcDate.hours   = atol(argv[3]);
    ls.utcDate.minutes = atol(argv[4]);
    ls.utcDate.seconds = atol(argv[5]);
    ls.utcDate.doy     = atol(argv[6]);
    ls.utcDate.years   = atol(argv[7]);

    printf("\n");
    printf(" Leap Second: %+d sec at %02u:%02u:%02u %03u/%04u UTC\n",
           ls.offset, ls.utcDate.hours, ls.utcDate.minutes, ls.utcDate.seconds,
           ls.utcDate.doy, ls.utcDate.years);

    err = TSYNC_CS_setLeapSec(hnd, &ls);
        
    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
