#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle         hnd;
    int                       rc;
    int                       ret     = 0;
    int                       devIdx;
    char                     *devName = DEFAULT_DEV;
    char                      fullDevName[32];
    char                     *pgmname = argv[0];
    TSYNC_ERROR               err     = TSYNC_SUCCESS;
    TSYNC_TimeScaleOffsetObj  ts;
    
    
    /* If invalid number of arguments... */
    if (argc < 4)
    {
        printf(" Usage: CS_SetTimeScaleOff <device index> <time scale> <time scale offset>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Time message */
    ts.scale  = (ML_TIME_SCALE)atoi(argv[2]);
    ts.offset = atoi(argv[3]);
    
    printf("\n");
    printf(" Time Scale Offset for ");

    switch ((ts.scale))
    {
        case ML_TIME_SCALE_UTC:   printf("UTC: ");     break;
        case ML_TIME_SCALE_TAI:   printf("TAI: ");     break;
        case ML_TIME_SCALE_GPS:   printf("GPS: ");     break;
        case ML_TIME_SCALE_LOCAL: printf("LOCAL: ");   break;
        default:                  printf("Unknown: "); break;
    }

    printf("%d\n", (ts.offset));

    /* Set Time scale offset */
    err = TSYNC_CS_setTimeScaleOff(hnd, &ts);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        ret = (strstr(tsync_strerror(err), "RC_CHG_PENDING") != NULL) ? 2 : 1;
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
