#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    TSYNC_FileOptionObj fileOption;
    int index;


    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: DCS_GetOptions <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Send Get Local transaction
    err = TSYNC_DCS_getOptions(hnd, &fileOption);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%hhu ", fileOption.length);
        for (index=0; index<fileOption.length; index++)
            printf("%hhu ", fileOption.type[index]);
        printf("\n");
    }

    printf("\n  DCS Get Options (%d) available Licenses :\n", fileOption.length);
    for (index=0; index<fileOption.length; index++)
    {
        switch(fileOption.type[index])
        {
            case DCS_MULTIGNSS  : printf("\n    License %d: MULTIGNSS\n",     index); break;
            case DCS_SKYLIGHT   : printf("\n    License %d: SKYLIGHT\n",      index); break;
            case DCS_TIMEKEEPER : printf("\n    License %d: TIMEKEEPER\n",    index); break;
            case DCS_PLLLOCK    : printf("\n    License %d: PLL LOCK\n",      index); break;
            case DCS_TKPTPV1    : printf("\n    License %d: TK PTPv1\n",      index); break;
            case DCS_AGPS       : printf("\n    License %d: AGPS Server\n",   index); break;
            case DCS_PLL2       : printf("\n    License %d: PLL2 LOCK\n",     index); break;
            case DCS_PD1        : printf("\n    License %d: PeerD v1\n",      index); break;
            case DCS_SEM        : printf("\n    License %d: SEM\n",           index); break;
            case DCS_ELORAN1    : printf("\n    License %d: E-Loran\n",       index); break;
            case DCS_BSH        : printf("\n    License %d: BroadShield\n",   index); break;
            case DCS_HQF        : printf("\n    License %d: HaveQuick Off\n", index); break;
            case DCS_TS1        : printf("\n    License %d: GPI Timestamp\n", index); break;
            case DCS_BSL        : printf("\n    License %d: BroadShield Pro\n", index); break;
            case DCS_BSD        : printf("\n    License %d: BroadShield Debug\n", index); break;

            default : printf("License Type (%d) unknown\n", fileOption.type[index]);
        }
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
