#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    unsigned int        index;
    TSYNC_GplParentData data;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GPR_GetStatistics <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Message
    index = atoi(argv[2]);

    // Send Transaction
    err = TSYNC_GPR_getParentData(hnd, index, &data);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i 0x%2X %i 0x%4X %i %i ",
                data.gmCid.cid[0],
                data.gmCid.cid[1],
                data.gmCid.cid[2],
                data.gmCid.cid[3],
                data.gmCid.cid[4],
                data.gmCid.cid[5],
                data.gmCid.cid[6],
                data.gmCid.cid[7],
                data.gmPortid,
                data.gmQual.clockAcc,
                data.gmQual.clockClass,
                data.gmQual.oSLV,
                data.gmpri.priority1,
                data.gmpri.priority2);

        printf("%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i %i %i %i\n",
                data.parentCid.cid[0],
                data.parentCid.cid[1],
                data.parentCid.cid[2],
                data.parentCid.cid[3],
                data.parentCid.cid[4],
                data.parentCid.cid[5],
                data.parentCid.cid[6],
                data.parentCid.cid[7],
                data.parentPortId,
                data.parentStatsCalc,
                data.parentObsCPCR,
                data.parentObsOSLV);
    }

    printf("\n");
    printf("GPR %i Parent Data: \n", index);
    printf("    GM Clock Identity: %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
                        // Clock Identity is displayed similar to a MAC Address.
                        data.gmCid.cid[0],
                        data.gmCid.cid[1],
                        data.gmCid.cid[2],
                        data.gmCid.cid[3],
                        data.gmCid.cid[4],
                        data.gmCid.cid[5],
                        data.gmCid.cid[6],
                        data.gmCid.cid[7]);
    printf("     GM Port Identity: %i\n",data.gmPortid);
    printf("          GM Accuracy: 0x%2X\n",data.gmQual.clockAcc);
    printf("       GM Clock Class: %i\n",data.gmQual.clockClass);
    printf("              GM OSLV: 0x%4X\n",data.gmQual.oSLV);
    printf("        GM Priority 1: %i\n",data.gmpri.priority1);
    printf("        GM Priority 2: %i\n",data.gmpri.priority2);
    printf("\n");
    printf("Parent Clock Identity: %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
                        // Clock Identity is displayed similar to a MAC Address.
                        data.parentCid.cid[0],
                        data.parentCid.cid[1],
                        data.parentCid.cid[2],
                        data.parentCid.cid[3],
                        data.parentCid.cid[4],
                        data.parentCid.cid[5],
                        data.parentCid.cid[6],
                        data.parentCid.cid[7]);
    printf("Parent Port Identity: %i\n",data.parentPortId);
    printf(" Parent Stats Valid?: %i\n",data.parentStatsCalc);
    printf("Parent Observed CPCR: %i\n",data.parentObsCPCR);
    printf("Parent Observed OSLV: %i\n",data.parentObsOSLV);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
