#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle     hnd;
    int                   rc;
    int                   ret     = 0;
    int                   devIdx;
    char                 *devName = DEFAULT_DEV;
    char                  fullDevName[32];
    char                 *pgmname = argv[0];
    TSYNC_ERROR           err     = TSYNC_SUCCESS;
    unsigned int          index;
    TSYNC_GplSlaveStats   stats;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GPR_GetSlaveStats <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Get Clock Identity Message
    index = atoi(argv[2]);

    // Send Get Number of Instances Transaction
    err = TSYNC_GPR_getSlaveStats(hnd, index, &stats);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
    	printf("%i %i %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i %03i.%03i.%03i.%03i %02X:%02X:%02X:%02X:%02X:%02X %i %i %i %i %i %i %i %i %i \n",
    	        stats.slaveNum,
				stats.ptpEn, 
    	        stats.cid[0],
    	        stats.cid[1],
    	        stats.cid[2],
    	        stats.cid[3],
    	        stats.cid[4],
    	        stats.cid[5],
    	        stats.cid[6],
    	        stats.cid[7],
    	        stats.pid,
    	        stats.ipV4Addr[0],
    	        stats.ipV4Addr[1],
    	        stats.ipV4Addr[2],
    	        stats.ipV4Addr[3],
				stats.macAddr[0],
    	        stats.macAddr[1],
    	        stats.macAddr[2],
    	        stats.macAddr[3],
    	        stats.macAddr[4],
    	        stats.macAddr[5],
    	        stats.syncPeriod,
    	        stats.annPeriod,
    	        stats.dRqPeriod,
    	        stats.syncDur,
				stats.syncRem,
    	        stats.annDur,
				stats.annRem,
    	        stats.dRqDur,
    	        stats.dRqRem);
    }

    printf("\n");
    printf(" GPR%i Slave Stats\n", (index));
    printf("      PTP Enable : %i\n", stats.ptpEn);          
    printf("    Slave Number : %i\n", stats.slaveNum);
    printf("  Clock Identity : %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
    	                // Clock Identity is displayed similar to a MAC Address.
    	                stats.cid[0],
    	                stats.cid[1],
    	                stats.cid[2],
    	                stats.cid[3],
    	                stats.cid[4],
    	                stats.cid[5],
    	                stats.cid[6],
    	                stats.cid[7]);
    printf("          Port ID: %i\n", stats.pid);
    printf("          IP Addr: %03i.%03i.%03i.%03i\n", stats.ipV4Addr[0],             
                                            stats.ipV4Addr[1],
                                            stats.ipV4Addr[2],
                                            stats.ipV4Addr[3]);
	 printf("  MAC Address : %02X:%02X:%02X:%02X:%02X:%02X\n",            
    	                // Clock Identity is displayed similar to a MAC Address.      
    	                stats.macAddr[0],                                                 
    	                stats.macAddr[1],                                                 
    	                stats.macAddr[2],                                                 
    	                stats.macAddr[3],                                                 
    	                stats.macAddr[4],                                                 
    	                stats.macAddr[5]);    
						
    printf("      Sync Period: %i\n", stats.syncPeriod);
    printf("       Ann Period: %i\n", stats.annPeriod);
    printf("       DRq Period: %i\n", stats.dRqPeriod);
    printf("    Sync Duration: %i\n", stats.syncDur);
    printf("     Ann Duration: %i\n", stats.annDur);
    printf("     DRq Duration: %i\n", stats.dRqDur);
    printf("   Sync Remaining: %i\n", stats.syncRem);
    printf("    Ann Remaining: %i\n", stats.annRem);
    printf("    DRq Remaining: %i\n", stats.dRqRem);



    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
