#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_GplUnctSlaveProp  slaveProp;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GPR_GetUnctSlaveProp <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Get Clock Identity Message
    index = atoi(argv[2]);

    // Send Get Number of Instances Transaction
    err = TSYNC_GPR_getUnctSlaveProp(hnd, index, &slaveProp);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %s %i %i %i %s %i %i %i %s %i %i %i \n",
                slaveProp.negoEnabled,
                slaveProp.contractAnnState,
                slaveProp.durationAnnContracts,
                slaveProp.delayAnnContracts,
                slaveProp.logAnnMsgInterval,
                slaveProp.contractSyncState,
                slaveProp.durationSyncContracts,
                slaveProp.delaySyncContracts,
                slaveProp.logSyncMsgInterval,
                slaveProp.contractDelayRespState,
                slaveProp.durationDelayRespContracts,
                slaveProp.delayDelayRespContracts,
                slaveProp.logDelayRespMsgInterval);
    }

    printf("\n");
    printf(" GPR%i Slave Properties: \n", (index));
    printf(" Negotiation Enabled          : %i \n", slaveProp.negoEnabled);
    printf(" Announce Contract State      : %s \n",slaveProp.contractAnnState);
    printf(" Announce Duration            : %i \n",slaveProp.durationAnnContracts);
    printf(" Announce Delay               : %i \n",slaveProp.delayAnnContracts);
    printf(" Log Announce Interval        : %i \n",slaveProp.logAnnMsgInterval);
    printf(" Sync contract State          : %s \n",slaveProp.contractSyncState);
    printf(" Sync Duration                : %i \n",slaveProp.durationSyncContracts);
    printf(" Sync Delay                   : %i \n",slaveProp.delaySyncContracts);
    printf(" Log Sync Interval            : %i \n",slaveProp.logSyncMsgInterval);
    printf(" Delay Response Contract State: %s \n",slaveProp.contractDelayRespState);
    printf(" Delay Response Duration      : %i \n",slaveProp.durationDelayRespContracts);
    printf(" Delay Response Delay         : %i \n",slaveProp.delayDelayRespContracts);
    printf(" Delay Response Interval      : %i \n",slaveProp.logDelayRespMsgInterval);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
