#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_GplMacAddr   macAddr;

    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: GPR_SetMacAddr <device index> <index> <XX:XX:XX:XX:XX:XX>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Set Clock Identity Message
    index = atoi(argv[2]);
    sscanf(argv[3],"%2hhx:%2hhx:%2hhx:%2hhx:%2hhx:%2hhx",
                   &(macAddr.macAddr[0]),     &(macAddr.macAddr[1]),
                   &(macAddr.macAddr[2]),     &(macAddr.macAddr[3]),
                   &(macAddr.macAddr[4]),     &(macAddr.macAddr[5]));

    // Send Set Transaction
    err = TSYNC_GPR_setMacAddr(hnd, index, &macAddr);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%02X:%02X:%02X:%02X:%02X:%02X\n",
                        macAddr.macAddr[0],
                        macAddr.macAddr[1],
                        macAddr.macAddr[2],
                        macAddr.macAddr[3],
                        macAddr.macAddr[4],
                        macAddr.macAddr[5]);
    }

    printf("\n");
    printf(" GPR%i Mac Address: %02X:%02X:%02X:%02X:%02X:%02X\n",
                        (index),
                        macAddr.macAddr[0],
                        macAddr.macAddr[1],
                        macAddr.macAddr[2],
                        macAddr.macAddr[3],
                        macAddr.macAddr[4],
                        macAddr.macAddr[5]);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
