#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_GplMsgTo     msgTo;

    /* If invalid number of arguments... */
    if (argc != 6)
    {
        printf(" Usage: GPR_SetMsgTo <device index> <index> <Ann TO> <Sync TO> <Delay Req TO>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Set Clock Identity Message
    index = atoi(argv[2]);
    msgTo.annRcptTimeout   = atoi(argv[3]);
    msgTo.syncTimeout      = atoi(argv[4]);
    msgTo.delayRespTimeout = atoi(argv[5]);

    // Send Set Transaction
    err = TSYNC_GPR_setMsgTo(hnd, index, &msgTo);
    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %i %i \n", msgTo.annRcptTimeout,
                msgTo.syncTimeout,
                msgTo.delayRespTimeout);
    }

    printf("\n");
    printf(" GPR%i Message Timeouts : \n", index);
    printf(" Announce Timeout      : %i \n", msgTo.annRcptTimeout);
    printf(" Sync Timeout          : %i \n", msgTo.syncTimeout);
    printf(" Delay Request Timeout : %i \n", msgTo.delayRespTimeout);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
