#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

// Function Prototypes
void makeSatIdStr(char* pSatIdStr, unsigned int satId);

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    char               satIdStr[8] = {0};
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_SatDataObj   sd;
    unsigned int       i;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GR_GetSatData <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Sat Data message */
    index = atoi(argv[2]);

    // Send Get Sat Data transaction
    err = TSYNC_GR_getSatData(hnd, index, &sd);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
     /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        // No Set Response is supported for this get command
        // Send spaces and linefeed in case line feeds are discarded
        printf("    \n");
    }

    printf("\n");
    printf(" GR (%d) Sat Data:\n", (index));
    printf(" ##:  ch  id   st  t  f  fl\n");
    printf(" --------------------------\n");

    for (i = 0; i < 32; i++)
    {
        TSYNC_GR_svidToString(satIdStr, sd.info[i].svid);

        printf(" %02d:  %02u  %s %02u  %01d  %01d  %03X\n", i,
               (sd.info[i].chnum),  satIdStr,
               (sd.info[i].str),    (sd.info[i].bTraim),
               (sd.info[i].bInfix), (sd.info[i].flags));
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */

