#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    unsigned int       constellation;

    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: GR_SetConstSel <device index> <index> <constellation>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Offset message */
    index         = atoi(argv[2]);
    constellation = atoi(argv[3]);

    printf("\n  GR (%d) ConstellationSelection: (%d) : \n", (index), (constellation));

    if ((constellation & GR_GPS_CONST) == GR_GPS_CONST)
        printf("GPS\n");

    if ((constellation & GR_GLONASS_CONST) == GR_GLONASS_CONST)
        printf("GLONASS\n");

    if ((constellation & GR_BEIDOU_CONST) == GR_BEIDOU_CONST)
        printf("BEIDOU\n");

    if ((constellation & GR_GALILEO_CONST) == GR_GALILEO_CONST)
        printf("GALILEO\n");

    if ((constellation & GR_SBAS_CONST) == GR_SBAS_CONST)
        printf("SBAS\n");

    if ((constellation & GR_QZSS_CONST) == GR_QZSS_CONST)
        printf("QZSS\n");

    // Send Set Offset transaction
    err = TSYNC_GR_setConstSel(hnd, index, constellation);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
