#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    GL_DYN             dyn;
    char               dynStr[32] = {0};


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: GR_SetDyn <device index> <index> <dyn>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Dynamics message */
    index = atoi(argv[2]);
    dyn   = (GL_DYN)atoi(argv[3]);

    switch (dyn)
    {
        case GL_DYN_LAND:   sprintf(dynStr, "LAND");       break;
        case GL_DYN_SEA:    sprintf(dynStr, "SEA");        break;
        case GL_DYN_AIR:    sprintf(dynStr, "AIR");        break;
        case GL_DYN_STAT:   sprintf(dynStr, "STATIONARY"); break;
        case GL_DYN_AIR_1G: sprintf(dynStr, "AIR1G");      break;
        case GL_DYN_AIR_4G: sprintf(dynStr, "AIR4G");      break;
        default:            sprintf(dynStr, "UNKNOWN");    break;
    }

    printf("\n  GR (%d) Dynamics Mode: %s (%d)\n", (index), dynStr, (dyn));

    // Send Set Dynamics transaction
    err = TSYNC_GR_setDynamics(hnd, index, dyn);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
