#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    TSYNC_HWTimeObj    hwTime;

    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: HW_GetTime <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Send Get Time message */
    err = TSYNC_HW_getTime(hnd, &hwTime);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d %d %d %d %d %d %d\n", hwTime.time.years, hwTime.time.doy,
                hwTime.time.hours, hwTime.time.minutes, hwTime.time.seconds,
                hwTime.time.ns, hwTime.bSync);
     }

    printf("\n");
    printf(" DOY Time:\n");
    printf("  Year: %04d\n", hwTime.time.years);
    printf("  DOY:  %03d\n", hwTime.time.doy);
    printf("  Hr:   %02d\n", hwTime.time.hours);
    printf("  Min:  %02d\n", hwTime.time.minutes);
    printf("  Sec:  %02d\n", hwTime.time.seconds);
    printf("  Nsec: %09d\n", hwTime.time.ns);
    printf("  Sync: %s\n", (hwTime.bSync) ? "TRUE" : "FALSE");

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
