#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle    hnd;
    int                  rc;
    int                  ret     = 0;
    int                  devIdx;
    char                *devName = DEFAULT_DEV;
    char                 fullDevName[32];
    char                *pgmname = argv[0];
    TSYNC_ERROR          err     = TSYNC_SUCCESS;
    TMSTMP_SRC           src;
    TSYNC_HWTimeDataObj  tsData;
    unsigned int         i;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: HW_GetTsData <device index> <src>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*Build Get Timestamp Data message */
    src = (TMSTMP_SRC)atoi(argv[2]);

    /* Send Get Timestamp Data transaction */
    err = TSYNC_HW_getTsData(hnd, src, &tsData);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    printf("\n");
    printf(" Timestamp source %d timestamp data:\n", (src));

    for (i = 0; i < TSYNC_TIMESTAMP_DATA_NUM; i++)
    {
        if (tsData.data[i].time.doy == 0)
        {
            break;
        }

        printf("  %03u, %04u %02u:%02u:%02u.%09u, %s\n",
               tsData.data[i].time.doy, tsData.data[i].time.years,
               tsData.data[i].time.hours, tsData.data[i].time.minutes,
               tsData.data[i].time.seconds, tsData.data[i].time.ns,
               (tsData.data[i].bSync) ? "Sync" : "No Sync");
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
