#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            pgno    = 0;
    TSYNC_InitStatusResult  statPage;
    int                     more    = 1;
    unsigned int            i;


    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: GetStatus <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    printf("\n");
    printf("  Module                           | Result\n");
    printf("  -----------------------------------------\n");

    while (more == 1)
    {
        // Send get status transaction
        err = TSYNC_IN_getStatus(hnd, pgno, &statPage);

        if (err != TSYNC_SUCCESS)
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }

        // Extract capability info entries
        for (i = 0; i < TSYNC_INIT_RESULT_NUM; i++)
        {
            if (statPage.results[i].module[0] == '\0')
            {
                break;
            }

            printf("  ");
            printf("%-32s | ", statPage.results[i].module);
            (statPage.results[i].result == RC_NO_ERROR) ? printf("PASS\n") :
                                                          printf("FAIL\n");
        }

        more = (statPage.more);
        pgno = (((pgno) + 1));
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
