#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    IL_CF              cf;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: IR_SetCtrlField <device index> <index> <cf>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Control Field message */
    index = atoi(argv[2]);
    cf    = (IL_CF)atoi(argv[3]);

    printf("\n  IR (%d) Control Field: (%d) ", (index), (cf));

    switch ((cf))
    {
        case IL_CF_200_04:      printf("RCC 200-04\n");         break;
        case IL_CF_1344:        printf("IEEE C37.118-2005\n");  break;
        case IL_CF_SPEC:        printf("Spectracom\n");         break;
        case IL_CF_SPEC_FAA:    printf("Spectracom FAA\n");     break;
        case IL_CF_NASA:        printf("NASA\n");               break;
        case IL_CF_1344_SPEC:   printf("Spectracom IEEE C37.118-2005\n");  break;
        case IL_CF_UNKNOWN:
        default:                printf("Unknown\n");            break;
    }

    // Send Set Control Field transaction
    err = TSYNC_IR_setCtrlField(hnd, index, cf);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
