#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    IL_FMT             fmt;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: IR_SetFormat <device index> <index> <fmt>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index = atoi(argv[2]);
    fmt   = (IL_FMT)atoi(argv[3]);

    printf("\n  IR (%d) Format: (%d) ", (index), (fmt));

    switch ((fmt))
    {
        case IL_FMT_IRIG_A:     printf("IRIG A\n");             break;
        case IL_FMT_IRIG_B:     printf("IRIG B\n");             break;
        case IL_FMT_IRIG_G:     printf("IRIG G\n");             break;
        case IL_FMT_NASA_36:    printf("NASA-36\n");            break;
        case IL_FMT_IRIG_E_100: printf("IRIG E (100 Hz)\n");    break;
        case IL_FMT_IRIG_E_1K:  printf("IRIG E (1 kHz)\n");     break;
        case IL_FMT_AUTO:
        case IL_FMT_UNKNOWN:
        default:                printf("Unknown\n");            break;
    }

    // Send Set Format transaction
    err = TSYNC_IR_setFormat(hnd, index, fmt);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
