#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPParentPropObj   parentPropObj;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetParentProp <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Parent Prop message */
    index = atoi(argv[2]);

    // Send Get Parent Prop transaction
    err = TSYNC_PTR_getParentProperties(hnd, index, &parentPropObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i %i %04X %08X\n",
                // Clock Identity is displayed similar to a MAC Address.
                parentPropObj.clockIdentity[0],
                parentPropObj.clockIdentity[1],
                parentPropObj.clockIdentity[2],
                parentPropObj.clockIdentity[3],
                parentPropObj.clockIdentity[4],
                parentPropObj.clockIdentity[5],
                parentPropObj.clockIdentity[6],
                parentPropObj.clockIdentity[7],
                parentPropObj.portNumber,
                parentPropObj.statsCalculated,
                parentPropObj.observedOSLV,    // Only Hex Value displayed here
                parentPropObj.observedCPCR);   // Only Hex Value displayed here

    }

    printf("\n");
    printf(" PTR (%d) Parent Properties information:\n", (index));
    printf("  Clock Identity   : %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
        parentPropObj.clockIdentity[0],
        parentPropObj.clockIdentity[1],
        parentPropObj.clockIdentity[2],
        parentPropObj.clockIdentity[3],
        parentPropObj.clockIdentity[4],
        parentPropObj.clockIdentity[5],
        parentPropObj.clockIdentity[6],
        parentPropObj.clockIdentity[7]);
    printf("  Port Number      : %i\n",  parentPropObj.portNumber);
    printf("  Stats Calculated : %i\n",  parentPropObj.statsCalculated);
    printf("  Observed OSLV    : %i (0x%4x)\n",  parentPropObj.observedOSLV,
                                                 parentPropObj.observedOSLV);
    printf("  Observed CPCR    : %i (0x%8x)\n",  parentPropObj.observedCPCR,
                                                 parentPropObj.observedCPCR);





    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
