#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPUnitSettingsObj unitSetObj;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetUnitSettings <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Unit Settings message */
    index = atoi(argv[2]);

    // Send Get Unit Settings transaction
    err = TSYNC_PTR_getUnitSettings(hnd, index, &unitSetObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i %i %i %i %i\n",
                unitSetObj.clockIdentity[0],
                unitSetObj.clockIdentity[1],
                unitSetObj.clockIdentity[2],
                unitSetObj.clockIdentity[3],
                unitSetObj.clockIdentity[4],
                unitSetObj.clockIdentity[5],
                unitSetObj.clockIdentity[6],
                unitSetObj.clockIdentity[7],
                unitSetObj.oneStepMode,
                unitSetObj.domainNumber,
                unitSetObj.priority1,
                unitSetObj.priority2,
                unitSetObj.forcedHoldover);
    }

    printf("\n");
    printf(" PTR (%d) Unit Settings information:\n", (index));
    printf("  Clock Identity   : %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
              unitSetObj.clockIdentity[0],
              unitSetObj.clockIdentity[1],
              unitSetObj.clockIdentity[2],
              unitSetObj.clockIdentity[3],
              unitSetObj.clockIdentity[4],
              unitSetObj.clockIdentity[5],
              unitSetObj.clockIdentity[6],
              unitSetObj.clockIdentity[7]);
    printf("  One Step Mode    : %i\n",  unitSetObj.oneStepMode);
    printf("  Domain Number    : %i\n",  unitSetObj.domainNumber);
    printf("  Priority 1       : %i\n",  unitSetObj.priority1);
    printf("  Priority 2       : %i\n",  unitSetObj.priority2);
    printf("  Forced Holdover  : %i\n",  unitSetObj.forcedHoldover);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
