#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_PTPEthernetItfObj itfInfo;


    /* If invalid number of arguments... */
    if (argc != 7) 
    {
            printf(" Usage: PTR_SetEthernetITF <device index> <index> <DHCP Enable> [<IP addr> <netmask> <default gateway>]\n");
            return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Ethernet ITF message */
    index = atoi(argv[2]);
    itfInfo.dhcpEnabled = atoi(argv[3]);
    if (argc == 7)
    {
        sscanf(argv[4],"%3hhi.%3hhi.%3hhi.%3hhi",
                       &(itfInfo.staticIpAddr[0]),   &(itfInfo.staticIpAddr[1]),
                       &(itfInfo.staticIpAddr[2]),   &(itfInfo.staticIpAddr[3]));
        sscanf(argv[5],"%3hhi.%3hhi.%3hhi.%3hhi",
                       &(itfInfo.netMask[0]),        &(itfInfo.netMask[1]),
                       &(itfInfo.netMask[2]),        &(itfInfo.netMask[3]));
        sscanf(argv[6],"%3hhi.%3hhi.%3hhi.%3hhi",
                       &(itfInfo.defaultGateway[0]), &(itfInfo.defaultGateway[1]),
                       &(itfInfo.defaultGateway[2]), &(itfInfo.defaultGateway[3]));
    } else
    {
        memset(itfInfo.staticIpAddr,   0, sizeof(itfInfo.staticIpAddr));
        memset(itfInfo.netMask,        0, sizeof(itfInfo.netMask));
        memset(itfInfo.defaultGateway, 0, sizeof(itfInfo.defaultGateway));
    }
                           
    printf("\n");
    printf(" PTR (%d) Ethernet ITF information:\n", (index));
    printf("  DHCP Enabled : %i\n", itfInfo.dhcpEnabled);
    if (argc == 7)
    {
        printf("  IP Address   : %i.%i.%i.%i\n", itfInfo.staticIpAddr[0],
               itfInfo.staticIpAddr[1],  itfInfo.staticIpAddr[2],  itfInfo.staticIpAddr[3]);
        printf("  Netmask      : %i.%i.%i.%i\n", itfInfo.netMask[0],
               itfInfo.netMask[1],    itfInfo.netMask[2],    itfInfo.netMask[3]);
        printf("  Default GW   : %i.%i.%i.%i\n", 
               itfInfo.defaultGateway[0],  itfInfo.defaultGateway[1],  
               itfInfo.defaultGateway[2],  itfInfo.defaultGateway[3]);
    }
    
    // Send Set Ethernet ITF Info transaction
    err = TSYNC_PTR_setEthernetItf(hnd, index, &itfInfo);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
