#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_getAltitude
==========================================================================*/

unsigned char TPRO_getAltitude (TPRO_BoardObj *hnd, TPRO_AltObj *Altp)
{
  int err;

  AltObj Altitude;

  tpro_boolean_t boardSupportsGPS;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(Altp);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #include "tpro_legacy_altitude.h"

  CHECK(TPRO_boardSupportsGPSHardware(hnd, &boardSupportsGPS), TPRO_SUCCESS);
  if (boardSupportsGPS != TPRO_TRUE) {
      return (TPRO_INVALID_BOARD_TYPE_ERR);
  }
  
  /**
  ***  create I/O control
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_GET_ALTITUDE, &Altitude);
  if (err < 0)	return (TPRO_COMM_ERR);


  /**
  ***  fill-in users context
  **/
  {
    float temp;

    temp = (float) (Altitude.meters);
    temp /= (float) 10;
    Altp->meters  = temp;
  }

  return (TPRO_SUCCESS);
}
