#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_getLongitude
==========================================================================*/

unsigned char TPRO_getLongitude (TPRO_BoardObj *hnd, TPRO_LongObj *Longp)
{
  int err;

  LongObj Longitude;

  tpro_boolean_t boardSupportsGPS;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(Longp);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #define LONGOBJ_NAME Longp
  #include "tpro_legacy_getlongitude.h"
  #undef LONGOBJ_NAME

  CHECK(TPRO_boardSupportsGPSHardware(hnd, &boardSupportsGPS), TPRO_SUCCESS);
  if (boardSupportsGPS != TPRO_TRUE) {
      return (TPRO_INVALID_BOARD_TYPE_ERR);
  }
  
  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_GET_LONGITUDE, &Longitude);
  if (err < 0)	return (TPRO_COMM_ERR);

  /**
  ***  fill-in users context
  **/
  {
    float temp;

    temp = (float) (Longitude.minutes);
    temp /= 10000.0;
    Longp->degrees = Longitude.degrees;
    Longp->minutes = temp;
  }

  return (TPRO_SUCCESS);
}
