#include <sys/ioctl.h>
#include <stdio.h>

#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_getTime
==========================================================================*/

unsigned char TPRO_getTime (TPRO_BoardObj *hnd, TPRO_TimeObj *Timep)
{
  int err;
  
  TimeObj TimeContext;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(Timep);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #include "tpro_legacy_gettime.h"
  
  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_GET_TIME, (char *) &TimeContext);
  if (err == -1) {
    printf ("File Descriptor: %d\n", hnd->file_descriptor);
    printf ("Error: %d\n", err);
    return (TPRO_COMM_ERR);
  }

  /**
  ***  fill-in users context
  **/
  {
    double temp;

    temp = (double) (TimeContext.secsDouble);
    temp /= (double) 1000000.0;
    Timep->secsDouble = temp;
    Timep->seconds = (unsigned char) (Timep->secsDouble);

    Timep->minutes = TimeContext.minutes;
    Timep->hours   = TimeContext.hours;
    Timep->days    = TimeContext.days;

    Timep->flags   = TimeContext.flags;
  }

  return (TPRO_SUCCESS);
}
