#include <sys/file.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <errno.h>
#include <string.h>
#include <ddtpro.h>
#include <tpro.h>


#include "tpro_macros.h"

/*==========================================================================
  TPRO_open
==========================================================================*/

unsigned char TPRO_open (TPRO_BoardObj **hnd, char *deviceName)
{
  BoardObj *Board;

  /**
  ***  allocate board object
  **/

  Board = (BoardObj *) malloc (sizeof (BoardObj));
  if (Board == NULL) {
    return (TPRO_OBJECT_ERR);
  }


  /**
  ***  open device
  **/

  Board->file_descriptor = open (deviceName, O_RDWR);
  if (Board->file_descriptor < 0) {
      errorLog("TPRO_open: open(%s) failed: %s\n", deviceName, strerror(errno));
      free(Board);
      return (TPRO_HANDLE_ERR);
  }


  /**
  *** create open I/O control
  **/

  if (ioctl (Board->file_descriptor, IOCTL_TPRO_OPEN, Board) < 0) {
    free(Board);
    return (TPRO_COMM_ERR);
  }


  /**
  ***  initialize strings
  **/

  memset (Board->firmware, 0, sizeof (Board->firmware) / sizeof (unsigned char));
  memset (Board->FPGA, 0, sizeof (Board->FPGA) / sizeof (unsigned char));


  /**
  ***  copy driver version
  **/

  if (ioctl (Board->file_descriptor, IOCTL_TPRO_GET_DRIVER, Board->driver) < 0) {
    free(Board);
    return (TPRO_COMM_ERR);
  }
  

#if 0
  /**
  ***  get firmware
  **/

  if (ioctl (Board->file_descriptor, IOCTL_TPRO_GET_FIRMWARE, Board->firmware) < 0) {
    free(Board);
    return (TPRO_COMM_ERR);
  }


  /**
  ***  get fpga version
  **/

  if ( (Board->options == 0x7000) || (Board->options == 0x7100) ) {

    if (ioctl (Board->file_descriptor, IOCTL_TPRO_GET_FPGA, Board->FPGA) < 0) {
      free(Board);
      return (TPRO_COMM_ERR);
    }
  }
#endif

  /**
  ***  set user context pointer
  **/

  *hnd = (TPRO_BoardObj *) Board;

  return (TPRO_SUCCESS);
}
