#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tpro_date_time.h"
#include "tsync.h"

/*==========================================================================
  TPRO_setPropDelayCorr
==========================================================================*/

unsigned char TPRO_setPropDelayCorr (TPRO_BoardObj *hnd, int *us)
{
  int err;


  // make sure we have valid handle
  if (hnd == NULL) {
    return (TPRO_HANDLE_ERR);
  }

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #define MICROSECONDS_NAME us
  #include "tpro_legacy_setpropdelaycorr.h"
  #undef MICROSECONDS_NAME
  
  /**
  ***  verify good correction factor if specified
  **/
  
  if (us != NULL)
  {
      /** Compact PCI **/
      if ( (hnd->options == 0x7000) || (hnd->options == 0x7100) ) {
        if ( (*us < -999) || (*us > 999) ) return (TPRO_DELAY_PARM_ERR);
      }

      /** PCI **/
      if ( (*us < -1000) || (*us > 8999) ) return (TPRO_DELAY_PARM_ERR);
  }


  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_PROP_DELAY_CORR, us);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
