#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tpro_date_time.h"
#include "tsync.h"

/*==========================================================================
  TPRO_setTime
==========================================================================*/

unsigned char TPRO_setTime (TPRO_BoardObj *hnd, TPRO_TimeObj *Timep)
{
  unsigned char syncOption = 0;
  tpro_boolean_t boardSyncsToPulseOnly;

  int err;

  TimeObj TimeContext;


  // make sure we have valid handle
  if (hnd == NULL) {
    return (TPRO_HANDLE_ERR);
  }

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #include "tpro_legacy_settime.h"
  
  /**
  ***  set time option based on board options
  **/

  CHECK(TPRO_boardSyncsToPulseOnly(hnd, &boardSyncsToPulseOnly), TPRO_SUCCESS);
  if (boardSyncsToPulseOnly == TPRO_TRUE) {
    syncOption = 1;
  }

  CHECK(tpro_validateTime(hnd, Timep), TPRO_SUCCESS);

  /**
  ***  fill-in working context
  **/

  TimeContext.syncOption = syncOption;

  TimeContext.days    = Timep->days;
  TimeContext.hours   = Timep->hours;
  TimeContext.minutes = Timep->minutes;
  TimeContext.seconds = Timep->seconds;


  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_TIME, &TimeContext);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
