#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of arguments */
    if (argc != 2) 
    {
         printf (" Usage: ResetFirmware <device index>\n");
         return (1);
    }
    
    /* Construct the TSAT/TPRO device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    printf(" Resetting %s's firmware - Please wait........", deviceName); 
    fflush (stdout);
    
    /* Reset the TPRO/TSAT firmware */
    rv = TPRO_resetFirmware(pBoard);
    
    /* If unable to reset the TPRO/TSAT firmware... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Reset Firmware!! [%d]\n", rv);
        retVal = 1;
    }
    else
    {
        printf ("Success\n");
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
