#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_HeartObj  tproHeart = {0};
    unsigned char  rv;
    
    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 3) 
    {
       printf(" Usage: SetHeart <device index> <frequency in Hz>\n");
       return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /*
    ** Initialize heart object 
    */
    tproHeart.signalType = SIG_PULSE;
    tproHeart.outputType = SIG_JAM;
    tproHeart.frequency  = atof (argv[2]);
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board!! [%d]\n", rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT heartbeat */
    rv = TPRO_setHeartbeat(pBoard, &tproHeart);
    
    /* If unable to set the TPRO/TSAT heartbeat... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not set heartbeat!! [%d]\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device ... */
    if (rv != TPRO_SUCCESS) 
    {
       printf(" Could Not Close Board!! [%d]\n", rv);
       retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
