#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    int            ticks;
    TPRO_BoardObj *pBoard;
    unsigned char  rv;
    
    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 3) 
    {
        printf(" Usage: WaitHeartbeat <device index> <seconds>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Read the TPRO/TSAT ticks argument */
    sscanf(argv[2], "%10d", &ticks);
    ticks *= 100;

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Wait for heartbeat */
    printf (" Waiting on heartbeat interrupt....\n");
    rv = TPRO_waitHeartbeat (pBoard, &ticks);

    switch (rv) {

    case TPRO_SUCCESS: 
        printf(" Received Heartbeat Interrupt!!\n\n");
        break;

    case TPRO_TIMEOUT_ERR:
        printf(" Timeout Receiving Heartbeat!!\n\n");
        retVal = 1;
        break;

    case TPRO_COMM_ERR:
        printf(" Error Communicating with Timing Board!!\n\n");
        retVal = 1;
        break;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
